/*
 * MINIGUI - Harbour Win32 GUI library 
 *
 * Copyright 2016 Verchenko Andrey <verchenkoag@gmail.com>
 * Copyright 2016 Sergej Kiselev <bilance@bilance.lv>
*/

#include "minigui.ch"
#include "tsbrowse.ch"
 
// GetCellSize( oBrw, nRowPos, nCell, lColSpecHd ) // Coordinates and cell dimensions TsBrowse
// TableInfoSuperHeader(oBrw)  // Info: superhider table
// TableInfoCell(oBrw)         // Info: table columns width
// TableInfoSize(oBrw)         // Info: dimensions in the table
// TBrw_Create( ..... )

///////////////////////////////////////////////////////////////////////////////
// Coordinates and cell dimensions TsBrowse
Function GetCellSize( oBrw, nRowPos, nCell, lColSpecHd ) 
LOCAL nI, ix, nStartX := 0, oCol, cBrw, cForm
LOCAL nRow, nCol, nWidth, nHeight

   Default nRowPos    := oBrw:nRowPos, ;
           nCell      := oBrw:nCell, ;
           lColSpecHd := .F.

   cForm := oBrw:cParentWnd
   cBrw  := oBrw:cControlName
   oCol  := oBrw:aColumns[ nCell ]

   If oBrw:nFreeze > 0
      For nI := 1 To Min( oBrw:nFreeze , nCell - 1 )
         nStartX += oBrw:GetColSizes()[ nI ]
      Next
   EndIf

   For nI := oBrw:nColPos To nCell - 1
      nStartX += oBrw:GetColSizes()[ nI ]
   Next

   IF lColSpecHd                  //   SpecHeader
      nRow    := oBrw:nHeightHead + oBrw:nHeightSuper + If( oCol:l3DLook, 2, 0 )
      nCol    := nStartX + If( oCol:l3DLook, 2, 0 )
      nWidth  := oBrw:GetColSizes()[ nCell ] - If( oCol:l3DLook, 2, 1 )
      nHeight := oBrw:nHeightSpecHd - If( oCol:l3DLook, 1, -1 )
   else                           //   
      nRow    := nRowPos - 1
      nRow    := ( nRow * oBrw:nHeightCell ) + oBrw:nHeightHead + ;
                 oBrw:nHeightSuper + oBrw:nHeightSpecHd + If( oCol:l3DLook, 2, 0 )
      nCol    := nStartX + If( oCol:l3DLook, 2, 0 )
      nWidth  := oBrw:GetColSizes()[ nCell ] - If( oCol:l3DLook, 2, 0 )
      nHeight := oBrw:nHeightCell - If( oCol:l3DLook, 1, -1 )
   endif

   ix := GetControlIndex ( cBrw, cForm )     //     
   if _HMG_aControlContainerRow [ix] == -1   // TGETBOX !!!
      nRow += oBrw:nTop - 1
      nCol += oBrw:nLeft
   else
      nRow += _HMG_aControlRow [ix] - 1
      nCol += _HMG_aControlCol [ix]
   endif

   nRow    += oBrw:aEditCellAdjust[1]
   nCol    += oBrw:aEditCellAdjust[2]
   nWidth  += oBrw:aEditCellAdjust[3] + 2
   nHeight += oBrw:aEditCellAdjust[4] + 2

Return { nRow, nCol, nWidth, nHeight }

//////////////////////////////////////////////////////////////////////
FUNCTION TableInfoSuperHeader(oBrw) 
    LOCAL nI, cMsg, cRet, aDim, cTitle
 
    cMsg := "LargeFont() = " + XTOC(LargeFonts()) + CRLF + CRLF
    cMsg += Replicate( "-._.", 22 ) + CRLF + CRLF
    aDim := oBrw:aSuperHead
    IF LEN(aDim) > 0
       For nI := 1 To LEN(aDim)
          cRet := STR(nI) + ") - : " + HB_NtoS(aDim[nI,1]) + " : " + HB_NtoS(aDim[nI,2])
          cTitle := ALLTRIM(aDim[nI,3])
          cTitle := STRTRAN(cTitle,CHR(10),"")
          cTitle := STRTRAN(cTitle,CHR(13),"") 
          cTitle := STRTRAN(cTitle,"\n","") 
          cRet += ' , ' + cTitle 
          cMsg += cRet  + CRLF + CRLF
       Next 
    ELSE
      cMsg += 'No superhider in the table !' + CRLF + CRLF
    ENDIF 
    cMsg += "oBrw:nHeightSuper=" + HB_NtoS(oBrw:nHeightSuper) + CRLF + CRLF
    cMsg += Replicate( "-._.", 22 ) + CRLF + CRLF
    MsgInfo(cMsg,"Info: superhider table" + (oBrw:cAlias))
   
    RETURN Nil

//////////////////////////////////////////////////////////////////////
FUNCTION TableInfoCell(oBrw) 
    LOCAL nI, cMsg, cRet

    cMsg := "LargeFont() = " + XTOC(LargeFonts()) + CRLF + CRLF
    cMsg += Replicate( "-._.", 22 ) + CRLF + CRLF
    For nI := 1 To oBrw:nColCount()
       cRet := oBrw:aColumns[nI]:cHeading
       cRet := STRTRAN(cRet,CHR(10),"") 
       cRet := STRTRAN(cRet,CHR(13)," ") 
       cRet := STRTRAN(cRet,'-',"") 
       cMsg += STR(nI,2) + ' "' + cRet +'"  - ' 
       cMsg += HB_NtoS(oBrw:GetColSizes()[nI]) + CRLF 
    Next 
    cMsg += Replicate( "-._.", 22 ) + CRLF + CRLF
    MsgInfo(cMsg,"Info: table columns width "+ (oBrw:cAlias))
   
    RETURN Nil

//////////////////////////////////////////////////////////////////////
FUNCTION TableInfoSize(oBrw) 
    LOCAL cMsg

    cMsg := "LargeFont() = " + XTOC(LargeFonts()) + CRLF + CRLF
    cMsg += Replicate( "-._.", 22 ) + CRLF + CRLF
    cMsg += "1) Height = " + HB_NtoS(oBrw:nHeight) + CRLF
    cMsg += "2) HeightHead = " + HB_NtoS(oBrw:nHeightHead) + CRLF
    cMsg += "3) HeightSuper = " + HB_NtoS(oBrw:nHeightSuper) + CRLF
    cMsg += "4) HeightFoot = " + HB_NtoS(oBrw:nHeightFoot) + CRLF
    cMsg += "5) HeightSpecHd = " + HB_NtoS(oBrw:nHeightSpecHd) + CRLF
    cMsg += "6) HeightCell = " + HB_NtoS(oBrw:nHeightCell) + CRLF + CRLF
    cMsg += "Number of rows in the table = " + HB_NtoS(oBrw:nRowCount()) + CRLF + CRLF 
    cMsg += "GetHScrollBarHeight() = " + HB_NtoS(GetHScrollBarHeight()) + CRLF 
    cMsg += "GetVScrollBarWidth() = " + HB_NtoS(GetVScrollBarWidth()) + CRLF 
    cMsg += Replicate( "-._.", 22 ) + CRLF + CRLF
    MsgInfo(cMsg,"Info: dimensions in the table " + (oBrw:cAlias))

    RETURN Nil

///////////////////////////////////////////////////////////////////////////////////////////////////////////////
Function TBrw_Create( ControlName, ParentForm, nRow, nCol, nWidth, nHeight, uAlias, lCell, FontName, FontSize, Backcolor )
   Local oBrw, nHgt := 0, nWdt := 0, hWnd, aRect := {0,0,0,0}
   Local aImages := NIL, aHeaders := NIL, aWidths := NIL, bFields := NIL, Value := 1
   Local tooltip := NIL, change := NIL, bDblclick := NIL, aHeadClick := NIL, gotfocus := NIL, lostfocus := NIL
   Local Delete := .F., lNogrid := .F., aJust := NIL, HelpId := NIL
   Local bold := .F., italic := .F., underline := .F., strikeout := .F.
   Local break := .F., fontcolor := NIL
   Local lock := .F., nStyle := NIL, appendable := .F., readonly := NIL
   Local valid := NIL, validmessages := NIL, aColors := NIL, uWhen := NIL, nId := NIL, aFlds := NIL, cMsg := NIL
   Local lRePaint := .T., lEnum := .F., lAutoSearch := .F., uUserSearch := NIL
   Local lAutoFilter := .F., uUserFilter := NIL, aPicture := NIL, lTransparent := .F.
   Local uSelector := NIL, lAutoCol := .F., aColSel := NIL, lEditable := .F.

    Default nRow       := 0,                    ;
            nCol       := 0,                    ;
            nWidth     := 0,                    ;
            nHeight    := 0,                    ;
            lCell      := .T.,                  ;
            backcolor  := NIL,                  ;
            ParentForm := _HMG_ThisFormName,    ;
            FontName   := _HMG_DefaultFontName, ;
            FontSize   := _HMG_DefaultFontSize  

    hWnd := GetFormHandle(ParentForm)

    GetClientRect(hWnd, aRect)
    
    If nWidth  < 0                                         //  
       nWdt   := nWidth
       nWidth := 0
    EndIf
    
    If nWidth == 0                                         //  
       nWidth := aRect[3]
    EndIf
    
    If nHeight < 0                                         //  
       nHgt    := nHeight
       nHeight := 0
    EndIf
    
    If nHeight == 0                                        //  
       nHeight := aRect[4] - nRow
    EndIf
    
    If nHgt < 0; nHeight += nHgt                           //  
    EndIf
    
    If nWdt < 0; nWidth  += nWdt                           //  
    EndIf
    
    oBrw := _DefineTBrowse( ControlName,   ;
                            ParentForm,    ;
                            nCol,          ;
                            nRow,          ;
                            nWidth,        ;
                            nHeight,       ;
                            aHeaders,      ;
                            aWidths,       ;
                            bFields,       ;
                            value,         ;
                            fontname,      ;
                            fontsize,      ;
                            tooltip,       ;
                            change,        ;
                            bDblclick,     ;
                            aHeadClick,    ;
                            gotfocus,      ;
                            lostfocus,     ;
                            uAlias,        ;
                            Delete,        ;
                            lNogrid,       ;
                            aImages,       ;
                            aJust,         ;
                            HelpId,        ;
                            bold,          ;
                            italic,        ;
                            underline,     ;
                            strikeout,     ;
                            break,         ;
                            backcolor,     ;
                            fontcolor,     ;
                            lock,          ;
                            lCell,         ;
                            nStyle,        ;
                            appendable,    ;
                            readonly,      ;
                            valid,         ;
                            validmessages, ;
                            aColors,       ;
                            uWhen,         ;
                            nId,           ;
                            aFlds,         ;
                            cMsg,          ;
                            lRePaint,      ;
                            lEnum,         ;
                            lAutoSearch,   ;
                            uUserSearch,   ;
                            lAutoFilter,   ;
                            uUserFilter,   ;
                            aPicture,      ;
                            lTransparent,  ;
                            uSelector,     ;
                            lEditable,     ;
                            lAutoCol,      ;
                            aColSel )
    
    oBrw:nClrLine    := COLOR_GRID
    oBrw:nWheelLines := 1
    oBrw:lNoHScroll  := .T.
    oBrw:lNoMoveCols := .T.
    oBrw:lNoLiteBar  := .F.
    oBrw:lNoResetPos := .F.
    oBrw:lPickerMode := .F.              // usual date format
    oBrw:nFireKey    := VK_F4            // set key VK_F4, default is VK_F2

Return oBrw

////////////////////////////////////////////////////////////
Function TBrw_Show( oBrw, nDelta )   

   _EndTBrowse()

   If hb_IsObject(oBrw)
      TBrw_NoHoles( oBrw, nDelta )
      oBrw:nHeightHead -= 1
   EndIf

   Return Nil

////////////////////////////////////////////////////////////
Function TBrw_Obj( cTbrw, cForm )          
   Local oBrw, i

   Default cForm := _HMG_ThisFormName

   If ( i := GetControlIndex(cTBrw, cForm) ) > 0
      oBrw:= _HMG_aControlIds [ i ]
   EndIf

   Return oBrw

////////////////////////////////////////////////////////////
Function TBrw_NoHoles( oBrw, nDelta, lSet ) 
   Local nI, nK, nHeight, nHole

   DEFAULT nDelta := 1, lSet := .T.
    
   nHole := oBrw:nHeight - oBrw:nHeightHead - oBrw:nHeightSuper - ; 
            oBrw:nHeightFoot - oBrw:nHeightSpecHd - ;
            If( oBrw:lNoHScroll, 0, GetHScrollBarHeight() ) 

   nHole   -= ( Int( nHole / oBrw:nHeightCell ) * oBrw:nHeightCell )
   nHole   -= nDelta
   nHeight := nHole

   If lSet

       nI := If( oBrw:nHeightSuper  > 0, 1, 0 ) + ;
             If( oBrw:nHeightHead   > 0, 1, 0 ) + ;
             If( oBrw:nHeightSpecHd > 0, 1, 0 ) + ;
             If( oBrw:nHeightFoot   > 0, 1, 0 )
             
       If nI > 0                          //  
      
          nK := int( nHole / nI )         //  nI -   
      
          If oBrw:nHeightFoot   > 0
             oBrw:nHeightFoot   += nK
             nHole              -= nK
          EndIf
          If oBrw:nHeightSuper  > 0
             oBrw:nHeightSuper  += nK
             nHole              -= nK
          EndIf
          If oBrw:nHeightSpecHd > 0
             oBrw:nHeightSpecHd += nK
             nHole              -= nK
          EndIf
          If oBrw:nHeightHead   > 0
             oBrw:nHeightHead   += nHole
          EndIf
      
       Else             //  ,    tsb   nHole
      
          SetProperty(oBrw:cParentWnd, oBrw:cControlName, "Height", ;
          GetProperty(oBrw:cParentWnd, oBrw:cControlName, "Height") - nHole)
      
       EndIf
      
       oBrw:Display()

   EndIf

   Return nHeight
